<?php
namespace Ep\Woo\Plugin\Payment;

use Ak\Wp\Plugin\PluginInterface;
use GuzzleHttp\Client;
use GuzzleHttp\ClientInterface;

/**
 * Class Adyen
 *
 * @package Ep\Woo\Plugin\Payment
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/tecdoc
 */
class Adyen extends \WC_Payment_Gateway implements PluginInterface
{
    /** @var ClientInterface */
    protected $http;

    protected $merchant;
    protected $username;
    protected $password;
    protected $authorizeurl;
    protected $liburl;
    protected $block;
    protected $failure;
    protected $success;

    private $rendered_receipt = false;

    /**
     * Adyen constructor.
     */
    public function __construct()
    {
        if (!function_exists('get_option')) {
            return;
        }

        $this->id = 'adyen';
        $this->method_title = 'Adyen';
        $this->enabled = $this->get_option('enabled') === 'yes';

        $this->init_form_fields();
        $this->init_settings();

        $this->icon = apply_filters('ep_asset_url', 'img/adyen.png');
        $this->title = $this->get_option('title');
        $this->description = $this->get_option('description');
        $this->merchant = $this->get_option('merchant');
        $this->username = $this->get_option('username');
        $this->password = $this->get_option('password');
        $this->authorizeurl = $this->get_option('authorizeurl');
        $this->liburl = $this->get_option('liburl');

        $this->block = $this->get_option('block');
        $this->success = $this->get_option('success');
        $this->failure = $this->get_option('failure');

        if (!has_action("woocommerce_receipt_{$this->id}")) {
            add_action("woocommerce_receipt_{$this->id}", [$this, 'receipt_page']);
            add_action("woocommerce_update_options_payment_gateways_{$this->id}", [$this, 'process_admin_options']);
        }
    }

    /**
     * Dodaje akcije i filtere u WordPress.
     */
    public function attach(): void
    {
        add_filter('woocommerce_payment_gateways', function(array $gateways) {
            $gateways[] = self::class;
            return $gateways;
        });
    }

    protected function getHttpClient()
    {
        if (null === $this->http) {
            $this->http = new Client([
                'headers' => [
                    'Authorization' => 'Basic ' . base64_encode($this->username . ':' . $this->password),
                    'Content-Type' => 'application/json',
                ],
            ]);
        }
        return $this->http;
    }

    public function receipt_page($order_id)
    {
        if ('POST' === $_SERVER['REQUEST_METHOD']) {
            $http = $this->getHttpClient();
            $order = wc_get_order($order_id);
            $amount = [
                'value' => round($order->get_total('') * 100),
                'currency' => get_woocommerce_currency(),
            ];

            /** @noinspection PhpUnhandledExceptionInspection */
            $response = $http->request('POST', $this->authorizeurl, [
                'json' => [
                    'additionalData' => [
                        'card.encrypted.json' => $_POST['adyen-encrypted-data'],
                    ],
                    'amount' => $amount,
                    'reference' => $_SERVER['HTTP_HOST'] . '-' . $order_id,
                    'merchantAccount' => $this->merchant,
                ],
            ]);

            $body = $response->getBody()->getContents();
            $response = json_decode($body, true);

            if ('Authorised' === $response['resultCode']) {
                $order->update_status('processing', sprintf('PSP Reference: %s', $response['pspReference']));
                wc()->cart->empty_cart(true);
                $redirect = get_permalink($this->success);
            } else {
                $order->update_status('failed', sprintf('Razlog: %s', $response['refusalReason']));
                $redirect = get_permalink($this->failure);
            }

            wp_redirect($redirect);
            exit;
        }

        if ($this->rendered_receipt) {
            return;
        }

        ?>
        <script type="text/javascript" src="<?= $this->liburl; ?>"></script>

        <div class="row">
            <div class="col large-8">
                <?php
                if ($this->block) {
                    $block = get_post($this->block);
                    echo do_shortcode($block->post_content);
                }
                ?>
            </div>

            <div class="col large-4">
                <form method="post" action="" id="adyen-encrypted-form">
                    <p class="form-row form-row-wide">
                        <label for="<?= esc_attr($this->id); ?>-holderName">
                            <?= esc_html__('Vlasnik kartice', 'ep-woocommerce'); ?>
                            <span class="required">*</span>
                        </label>

                        <input id="<?= esc_attr($this->id); ?>-holderName" type="text" data-encrypted-name="holderName">
                    </p>

                    <p class="form-row form-row-wide">
                        <label for="<?= esc_attr($this->id); ?>-number">
                            <?= esc_html__('Card number', 'woocommerce'); ?>
                            <span class="required">*</span>
                        </label>

                        <input id="<?= esc_attr($this->id); ?>-number" type="text" data-encrypted-name="number">
                    </p>

                    <p class="form-row form-row-wide" style="display: flex; flex-flow: row wrap">
                        <label style="flex-basis: 100%;" for="<?= esc_attr($this->id); ?>-expiryMonth">
                            <?= esc_html__('Kartica vrijedi do', 'ep-woocommerce'); ?>
                            <span class="required">*</span>
                        </label>

                        <input style="flex-basis: 48%;margin-right: 4%;" id="<?= esc_attr($this->id); ?>-expiryMonth" type="text" placeholder="MM" size="2" data-encrypted-name="expiryMonth">
                        <input style="flex-basis: 48%;" type="text" size="4" placeholder="<?= __('GGGG', 'ep-woocommerce'); ?>" data-encrypted-name="expiryYear">
                    </p>

                    <p class="form-row form-row-wide">
                        <label for="<?= esc_attr($this->id); ?>-cvc">
                            <?= esc_html__('Kontrolni broj (CVC/CID)', 'ep-woocommerce'); ?>
                            <span class="required">*</span>
                        </label>

                        <input id="<?= esc_attr($this->id); ?>-cvc" type="text" data-encrypted-name="cvc">
                    </p>

                    <input type="hidden" value="<?= date('c'); ?>" data-encrypted-name="generationtime">

                    <p class="form-row form-row-last">
                        <input type="submit" value="<?= __('Pay', 'woocommerce'); ?>">
                    </p>
                </form>
            </div>
        </div>

        <!--suppress ES6ConvertVarToLetConst, JSUnresolvedVariable, JSUnresolvedFunction -->
        <script>
            var form = document.getElementById('adyen-encrypted-form');
            var options = {};
            adyen.createEncryptedForm(form, options);
        </script>
        <?php

        $this->rendered_receipt = true;
    }

    public function process_payment($order_id)
    {
        $order = wc_get_order($order_id);

        return [
            'result' => 'success',
            'redirect' => $order->get_checkout_payment_url(true),
        ];
    }

    public function init_form_fields()
    {
        if (!is_admin()) {
            return;
        }

        /** @var \WP_Post[] $blocks */
        $blocks = get_posts([
            'posts_per_page' => -1,
            'post_status' => 'publish',
            'post_type' => 'blocks',
        ]);

        /** @var \WP_Post[] $pages */
        $pages = get_posts([
            'posts_per_page' => -1,
            'post_status' => 'publish',
            'post_type' => 'page',
        ]);

        $blockOptions = ['' => '-'];
        foreach ($blocks as $block) {
            $blockOptions[$block->ID] = $block->post_title;
        }

        $pageOptions = ['' => '-'];
        foreach ($pages as $page) {
            $pageOptions[$page->ID] = $page->post_title;
        }

        $this->form_fields = [
            'enabled' => [
                'title' => 'Omogući/Onemogući',
                'type' => 'checkbox',
                'label' => 'Omogući Adyen naplatu',
                'default' => 'no',
            ],
            'title' => [
                'title' => 'Naziv',
                'type' => 'text',
                'description' => 'Ovo će korisnici vidjeti na stranici naplate',
                'default' => 'Kreditna kartica',
                'desc_tip' => true,
            ],
            'description' => [
                'title' => 'Opis',
                'type' => 'textarea',
            ],
            'block' => [
                'title' => 'Blok na stranici s s obrascem',
                'type' => 'select',
                'options' => $blockOptions,
            ],
            'success' => [
                'title' => 'Stranica nakon uspješne kupovine',
                'type' => 'select',
                'options' => $pageOptions,
            ],
            'failure' => [
                'title' => 'Stranica nakon greške prilikom naplate',
                'type' => 'select',
                'options' => $pageOptions,
            ],
            'merchant' => [
                'title' => 'Merchant account',
                'type' => 'text',
            ],
            'username' => [
                'title' => 'Korisničko ime',
                'type' => 'text',
                'description' => 'npr. ws@Company.YourCompany',
            ],
            'password' => [
                'title' => 'Lozinka',
                'type' => 'text',
            ],
            'authorizeurl' => [
                'title' => 'URL za autorizaciju naplate',
                'type' => 'text',
            ],
            'liburl' => [
                'title' => 'URL Adyen biblioteke za enkripciju',
                'type' => 'text',
            ],
        ];
    }
}