<?php
namespace Ep\Woo\Widget;

use Ep\TecDoc\TecDocService;
use Ep\Wp\Widget\AbstractWidget;
use Psr\Cache\CacheItemPoolInterface;
use WP_Taxonomy;
use WP_Term;

/**
 * Class Categories
 *
 * @package Ep\Woo\Widget
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-woocommerce
 */
class Categories extends AbstractWidget
{
    /** @var CacheItemPoolInterface */
    protected $cache;
    /** @var TecDocService */
    protected $tecdoc;

    public function __construct(CacheItemPoolInterface $cache, TecDocService $tecdoc = null)
    {
        $this->cache = $cache;
        $this->tecdoc = $tecdoc;

        parent::__construct('ep_categories', 'EP Kategorije', [
            'classname' => 'ep-widget-categories',
            'description' => 'Omogućuje odabir kategorije putem obrasca.',
        ]);

        add_action('wp_ajax_ep_taxonomy', [$this, 'ajaxTaxonomy']);
        add_action('wp_ajax_nopriv_ep_taxonomy', [$this, 'ajaxTaxonomy']);
    }

    /**
     * Šalje djecu roditelja iz postavljene taksonomije.
     */
    public function ajaxTaxonomy()
    {
        if (!isset($_POST['widget'])) {
            wp_die('Parametar `widget` nije primljen!');
        }

        if (!isset($_POST['parent'])) {
            wp_die('Parametar `parent` nije primljen!');
        }

        $index = absint(preg_replace('/^ep_categories-/', '', $_POST['widget']));
        $settings = $this->get_settings();

        if (!isset($settings[$index])) {
            wp_die(sprintf('Postavke za widget `%s` ne postoje!', $_POST['widget']));
        }

        $instance = $settings[$index];

        if (!$instance['taxonomy']) {
            wp_die('Ovaj widget nema postavljenu taksonomiju.');
        }

        $terms = $this->getTerms($instance, absint($_POST['parent']));

        wp_send_json($terms);
    }

    /**
     * Renderira widget.
     *
     * @param array $args
     * @param array $instance
     *
     * @throws \Psr\Cache\InvalidArgumentException
     */
    public function widget($args, $instance)
    {
        wp_enqueue_script('ep-core');
        $showImages = isset($instance['show_images'])  && '1' === $instance['show_images'];
        $autoSubmit = isset($instance['auto_submit']) && '1' === $instance['auto_submit'];
        $showButton = isset($instance['show_button']) && '1' === $instance['show_button'];
        $rootAsCurrent = isset($instance['root_as_current']) && '1' === $instance['root_as_current'];
        $labels = isset($instance['labels']) && !empty($instance['labels']) ? explode("\n", $instance['labels']) : [];

        if (!isset($instance['taxonomy'])) {
            return;
        }

        $cacheKeyPrefix = $_COOKIE['_tv'] ?? ($_COOKIE['_moto_id'] ?? '') ;
        $cacheKey = sprintf('widget.categories.%s', md5($cacheKeyPrefix . serialize($instance)));
        $cacheItem = $this->cache->getItem($cacheKey);

        if (!$cacheItem->isHit()) {
            ob_start();
            ?>
            <div id="<?= $this->id; ?>" data-autosubmit="<?= $autoSubmit; ?>" class="widget widget-ep-categories">
                <?php if ($instance['title']): ?>
                    <span class="widget-title" style="display:block;"><?= esc_html($instance['title']); ?></span>
                    <div class="is-divider small"></div>
                <?php endif; ?>

                <?php
                if (isset($instance['taxonomy'], $instance['widget_style']) && 'list' === $instance['widget_style']):
                    $root = $instance['root'];

                    if ('current' === $instance['root']) {
                        $root = is_archive() ? get_queried_object_id() : 0;

                        if ($rootAsCurrent && $root > 0) {
                            $ancestors = get_ancestors($root, $instance['taxonomy'], 'taxonomy');

                            if (count($ancestors) > 0) {
                                $root = array_shift($ancestors);
                            }
                        }
                    }

                    /** @var WP_Term[] $categories */
                    $categories = get_terms([
                        'taxonomy' => $instance['taxonomy'],
                        'parent' => intval($root),
                        'hide_empty' => false,

                    ]);
                    ?>
                    <ul class="product-categories">
                        <?php foreach ($categories as $category): ?>
                            <li class="cat-item">
                                <a href="<?= esc_attr(get_term_link($category->term_id)); ?>">
                                    <?php if($showImages):
                                        $thumbnailId = get_term_meta($category->term_id, 'thumbnail_id', true);
                                        $thumbnailUrl = wp_get_attachment_thumb_url($thumbnailId) ?: wc_placeholder_img_src();
                                        ?>
                                        <img src="<?= esc_attr($thumbnailUrl); ?>" alt="<?= esc_attr($category->name); ?>">
                                    <?php endif; ?>
                                    <?= esc_html($category->name); ?>
                                </a>
                            </li>
                        <?php endforeach; ?>
                    </ul>
                <?php
                elseif ($instance['taxonomy'] && $instance['depth'] > 0):
                    /**
                     * Sadrži id-eve kategorija u trenutnoj grani.
                     *
                     * @var int[] $currentIds
                     */
                    $currentIds = [];

                    if (is_archive()) {
                        /** @var WP_Term $currentTerm */
                        $currentTerm = get_queried_object();

                        if ($currentTerm instanceof WP_Term && $instance['taxonomy'] === $currentTerm->taxonomy) {
                            $currentIds = get_ancestors($currentTerm->term_id, $instance['taxonomy'], 'taxonomy');
                            array_unshift($currentIds, $currentTerm->term_id);
                            array_push($currentIds, 0);
                            $currentIds = array_reverse($currentIds);
                        }
                    }

                    $root = absint($instance['root']);

                    while (count($currentIds) > 0 && $currentIds[0] !== $root) {
                        array_shift($currentIds);
                    }

                    array_shift($currentIds);

                    for ($i = 0, $ii = absint($instance['depth']); $i < $ii; $i++):
                        ?>
                        <!--suppress HtmlFormInputWithoutLabel -->
                        <select>
                            <option value=""><?= isset($labels[$i]) ? esc_html($labels[$i]) : ''; ?></option>

                            <?php if (false !== $root):
                                foreach ($this->getTerms($instance, $root) as $term): ?>
                                    <option value="<?= $i + 1 === $ii ? $term['url'] : $term['id']; ?>"
                                            data-url="<?= $term['url'] ?>"
                                            <?= isset($currentIds[$i]) && $currentIds[$i] === $term['id'] ? 'selected' : ''; ?>
                                    >
                                        <?= $term['name']; ?>
                                    </option>
                                <?php endforeach;
                            endif; ?>
                        </select>
                        <?php

                        $root = isset($currentIds[$i]) ? $currentIds[$i] : false;
                    endfor;

                    if ($showButton): ?>
                        <button type="button" class="button primary">
                            <?= isset($instance['button_text']) ? esc_html($instance['button_text']) : 'Kreni!'; ?>
                        </button>
                    <?php endif;
                endif; ?>
            </div>
            <?php
            $rendered = ob_get_clean();

            $cacheItem->set($rendered);
            $this->cache->save($cacheItem);
        }

        echo $cacheItem->get();
    }

    /**
     * Vraća pojmove za roditelja.
     *
     * @param array $instance
     * @param int $parent
     * @return array
     */
    protected function getTerms(array $instance, $parent = 0)
    {
        /** @var WP_Term[] $terms */
        $terms = get_terms([
            'taxonomy' => $instance['taxonomy'],
            'parent' => $parent,
            'hide_empty' => isset($instance['hide_empty']),
        ]);

        return array_map(function(WP_Term $term) use ($instance) {
            $name = esc_html($term->name);

            if (isset($instance['show_count']) && '1' === $instance['show_count']) {
                $name = sprintf('%s (%d)', $name, $term->count);
            }

            return [
                'id' => $term->term_id,
                'url' => esc_attr(get_term_link($term)),
                'name' => $name,
            ];
        }, $terms);
    }

    /**
     * Renderira obrazac s postavkama.
     *
     * @param array $instance
     * @return string|void
     */
    public function form($instance)
    {
        $currentTaxonomy = isset($instance['taxonomy']) ? $instance['taxonomy'] : null;
        $rootTerm = isset($instance['root']) ? $instance['root'] : null;
        $depth = isset($instance['depth']) ? absint($instance['depth']) : 0;
        $widgetStyle = isset($instance['widget_style']) ? $instance['widget_style'] : 'form';
        ?>
        <p>
            <label for="<?= $this->get_field_id('title'); ?>">Naslov:</label>
            <input type="text"
                   id="<?= $this->get_field_id('title'); ?>"
                   name="<?= $this->get_field_name('title'); ?>"
                   class="widefat"
                   value="<?= isset($instance['title']) ? esc_attr($instance['title']) : ''; ?>">
        </p>

        <p>
            <label for="<?= $this->get_field_id('taxonomy'); ?>">Taksonomija:</label>
            <select id="<?= $this->get_field_id('taxonomy'); ?>"
                    name="<?= $this->get_field_name('taxonomy'); ?>"
                    class="widefat">
                <option value="">-</option>

                <?php
                /** @var WP_Taxonomy[] $wp_taxonomies */
                global $wp_taxonomies;
                $taxonomies = [];

                foreach ($wp_taxonomies as $name => $taxonomy) {
                    $taxonomies[$taxonomy->name] = $taxonomy->label;
                }

                asort($taxonomies);

                foreach ($taxonomies as $name => $label) {
                    /** @noinspection HtmlUnknownAttribute */
                    printf(
                        '<option value="%s" %s>%s</option>',
                        esc_attr($name),
                        $name === $currentTaxonomy ? ' selected' : '',
                        esc_html($label)
                    );
                }
                ?>
            </select>
        </p>

        <?php if ($currentTaxonomy): ?>
        <p>
            <label for="<?= $this->get_field_id('root'); ?>">Vršni pojam:</label>
            <select id="<?= $this->get_field_id('root'); ?>"
                    name="<?= $this->get_field_name('root'); ?>"
                    class="widefat">
                <option value="">-</option>
                <option value="current"<?= $rootTerm === 'current' ? ' selected' : ''; ?>>Trenutni pojam</option>

                <?php
                /** @var WP_Term[] $all_roots */
                $all_roots = get_terms([
                    'taxonomy' => $currentTaxonomy,
                    'parent' => 0,
                    'hide_empty' => false,
                ]);

                $roots = [];

                foreach ($all_roots as $root) {
                    $roots[$root->term_id] = $root->name;
                }

                asort($roots);

                foreach ($roots as $id => $name) {
                    /** @noinspection HtmlUnknownAttribute */
                    printf(
                        '<option value="%s" %s>%s</option>',
                        $id,
                        $id == $rootTerm ? ' selected' : '',
                        esc_html($name)
                    );
                }
                ?>
            </select>
        </p>

        <p>
            <input type="checkbox"
                   id="<?= $this->get_field_id('root_as_current'); ?>"
                   name="<?= $this->get_field_name('root_as_current'); ?>"
                   class="checkbox"
                   value="1"
                <?= isset($instance['root_as_current']) && $instance['root_as_current'] === '1' ? 'checked' : ''; ?>>
            <label for="<?= $this->get_field_id('root_as_current'); ?>">Za trenutni pojam ostani na prvoj razini</label>
        </p>

        <p>
            <label for="<?= $this->get_field_id('widget_style'); ?>">Stil widgeta:</label>
            <select id="<?= $this->get_field_id('widget_style'); ?>"
                    name="<?= $this->get_field_name('widget_style'); ?>"
                    class="widefat">
                <option value="form"<?= 'form' === $widgetStyle ? ' selected' : ''; ?>>Obrazac</option>
                <option value="list"<?= 'list' === $widgetStyle ? ' selected' : ''; ?>>Lista</option>
            </select>
        </p>

        <p>
            <input type="checkbox"
                   id="<?= $this->get_field_id('show_images'); ?>"
                   name="<?= $this->get_field_name('show_images'); ?>"
                   class="checkbox"
                   value="1"
                <?= isset($instance['show_images']) && $instance['show_images'] === '1' ? 'checked' : ''; ?>>
            <label for="<?= $this->get_field_id('show_images'); ?>">Prikaži slike povezane s pojmom</label>
        </p>

        <p>
            <label for="<?= $this->get_field_id('depth'); ?>">Dubina navigacije:</label>
            <input type="number"
                   id="<?= $this->get_field_id('depth'); ?>"
                   name="<?= $this->get_field_name('depth'); ?>"
                   class="widefat"
                   value="<?= $depth; ?>">
        </p>

        <p>
            <label for="<?= $this->get_field_id('labels'); ?>">Zadane oznake polja:</label>
            <textarea id="<?= $this->get_field_id('labels'); ?>"
                      name="<?= $this->get_field_name('labels'); ?>"
                      rows="4"
                      class="widefat"><?= isset($instance['labels']) ? esc_html($instance['labels']) : ''; ?></textarea>
        </p>

        <p>
            <input type="checkbox"
                   id="<?= $this->get_field_id('show_count'); ?>"
                   name="<?= $this->get_field_name('show_count'); ?>"
                   class="checkbox"
                   value="1"
                <?= isset($instance['show_count']) && $instance['show_count'] === '1' ? 'checked' : ''; ?>>
            <label for="<?= $this->get_field_id('show_count'); ?>">Prikaži broj postova u pojmu</label>
        </p>

        <p>
            <input type="checkbox"
                   id="<?= $this->get_field_id('hide_empty'); ?>"
                   name="<?= $this->get_field_name('hide_empty'); ?>"
                   class="checkbox"
                   value="1"
                <?= isset($instance['hide_empty']) && $instance['hide_empty'] === '1' ? 'checked' : ''; ?>>
            <label for="<?= $this->get_field_id('hide_empty'); ?>">Sakrij prazne pojmove</label>
        </p>

        <p>
            <input type="checkbox"
                   id="<?= $this->get_field_id('auto_submit'); ?>"
                   name="<?= $this->get_field_name('auto_submit'); ?>"
                   class="checkbox"
                   value="1"
                <?= isset($instance['auto_submit']) && $instance['auto_submit'] === '1' ? 'checked' : ''; ?>>
            <label for="<?= $this->get_field_id('auto_submit'); ?>">Automatski trigger</label>
        </p>

        <p>
            <input type="checkbox"
                   id="<?= $this->get_field_id('show_button'); ?>"
                   name="<?= $this->get_field_name('show_button'); ?>"
                   class="checkbox"
                   value="1"
                <?= isset($instance['show_button']) && $instance['show_button'] === '1' ? 'checked' : ''; ?>>
            <label for="<?= $this->get_field_id('show_button'); ?>">Prikaži gumb</label>
        </p>

        <p>
            <label for="<?= $this->get_field_id('button_text'); ?>">Oznaka gumba:</label>
            <input type="text"
                   id="<?= $this->get_field_id('button_text'); ?>"
                   name="<?= $this->get_field_name('button_text'); ?>"
                   class="widefat"
                   value="<?= isset($instance['button_text']) ? esc_attr($instance['button_text']) : ''; ?>">
        </p>
    <?php else: ?>
        <p>
            Odaberite taksonomiju i spremite postavke.
        </p>
    <?php endif;
    }
}